import {$on} from './helpers.js';
import {Store} from './store.js';

const controller = function() {

    const store = Store('todos-modstache');
    let _activeRoute = '';
    let model = {
        newTodoTitle: '',
        todos: [],
        filteredTodos: [],
        todoClass: ({root}) => (root.completed) ? "completed" : "",
        toggleCompleted: (ctx,e) => () => { 
            ctx.root.completed = e.checked; 
            ctx.base.todoClass = ctx.base.todoClass; 
            UpdateTodo(ctx.root);
            FilterTodos();
            UpdateView();  },
        addTodo: ({element}) => () => { model.newTodoTitle = element.value; AddNewTodo(); UpdateView(); },
        displayTodos: () => store.count() > 0 ? null : 'none',
        todoCount: () => { let count=Incomplete(); return (count === 1) ? (count + ' item left') : (count + ' items left') },
        removeTodo: ({root}) => () => { RemoveTodo(root); UpdateView(); },
        filterButtonClass: ({element}) => element.href.substr(element.href.indexOf('#')) == ('#/' + _activeRoute) ? 'selected' : '',
        initEdit: (ctx,e) => { 
            ctx.root.editControl = e; 
        },
        edit: (ctx,e) => () => { 
            const listItem = e.parentElement.parentElement;
            listItem.classList.add('editing');
            ctx.root.showEdit = null; 
            ctx.root.editControl.focus();
        },
        checkCancel: (ctx,e) => ({key}) =>  {
            if (key == 'Escape') { 
                const listItem = e.parentElement;
                listItem.classList.remove('editing');
                ctx.root.showEdit = 'none'; 
                e.value = ctx.root.title; 
            } 
        },
        checkSaveOnEnter: (ctx,e) => ({key}) => { 
            if (key == 'Enter') 
                model.saveEdit(ctx,e)(); 
        },
        saveEdit: (ctx,e) => () =>  {
            if (ctx.root.showEdit === null) { 
                const listItem = e.parentElement;
                listItem.classList.remove('editing');
                ctx.root.title = e.value; 
                ctx.root.showEdit = 'none'; 
                UpdateTodo(ctx.root); 
            } 
        },
        startEdit: (ctx,e) => () => { 
            if (ctx.root.showEdit === null) {
                 e.focus(); 
                } 
            return ctx.root.title; 
        },
        showClearCompleted: (ctx) => { 
            let count=Completed(); 
            return (count > 0) ? null : 'none';
        },
        clearCompleted: (ctx) => () => {
            ClearAllCompleted();
            FilterTodos();
            UpdateView();
        }
    };

    $on(window, 'load', Init);
    function Init() {
        $on(window, 'hashchange', SetFilter);
        SetRoute();
        _M_.fill(document.body, model);
        FilterTodos();
    }

    function UpdateView() {
        model.displayTodos = model.displayTodos;
        model.todoCount = model.todoCount;
        model.showClearCompleted = model.showClearCompleted;
    }

    function Incomplete()
    {
        return store.count() - store.completed();
    }

    function Completed()
    {
        return store.completed();
    }

    function ClearAllCompleted() {
        const todos = store.find((item) => item.completed);
        todos.forEach((item) => {
            store.delete(item);
        });
    }

    async function AddNewTodo() {
        const title = model.newTodoTitle.trim();
        if (title) {
            await store.insert({
                id: Date.now(),
                title,
                completed: false
            });
            ClearNewTodo();
            FilterTodos();
        }
    }

    function ClearNewTodo() {
        model.newTodoTitle = '';
    }

    function FilterTodos() {
        let filter = {
            '': () => true,
            'active': (item) => item.completed == false,
            'completed':  (item) => item.completed == true
        }[_activeRoute];
        let filtered = store.find(filter);
        filtered.forEach((item) => Object.assign(item, { showEdit: 'none' })); // add view state specific properties

        model.filteredTodos.length = 0;
        model.filteredTodos.push(...filtered); // show todo items
    }

    function RemoveTodo(item) {
        store.delete(item);
        FilterTodos();
    }

	function UpdateTodo(item) {
        store.update(item);
	}

    function SetRoute() {
		const route = document.location.hash.replace(/^#\//, '');
		_activeRoute = route;
    }

    function SetFilter() {
        SetRoute();
		FilterTodos();
        model.filterButtonClass = model.filterButtonClass;
    }
}();